<?php
/**
 * Plugin Advanced Mod Rewrite settings
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */


if(!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

global $cfg, $client, $load_client, $contenido;

if (isset($client) && (int) $client > 0) {
    $clientId = (int) $client;
} elseif (isset($load_client) && (int) $load_client > 0) {
    $clientId = (int) $load_client;
} else {
    $clientId = '';
}


// Include necessary sources
cInclude('classes', 'Debug/DebuggerFactory.class.php');
cInclude('classes', 'mp/class.mpdebug.php');
plugin_include('mod_rewrite', 'classes/class.modrewrite.php');
plugin_include('mod_rewrite', 'classes/class.modrewritecontroller.php');
plugin_include('mod_rewrite', 'includes/functions.mod_rewrite.php');
cInclude('classes', 'mp/class.confighandler.php');

// set debug configuration
$options = array(
    'enable'                    => false, // disable debug
    'ressource_urls'            => array('/contenido/logs/errorlog.txt'),
    'dump_super_globals'        => array('$_GET', '$_POST', '$_COOKIE', '$_SESSION'),
    'ignore_empty_superglobals' => true
);
$GLOBALS['mpDebug']->setConfig($options);


// load configuration
mr_loadConfiguration($clientId);


if (ModRewrite::is_enabled()) {

    if (isset($contenido)) {
        // we are in backend, don't register any cec functions and get out here
        return;
    }

    $aMrCfg = ModRewrite::getConfig();

    ////////////////////////////////////////////////////////////////////////////////////////////////

    $_cecRegistry = cApiCECRegistry::getInstance();

    // Add new tree function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_newtree.AfterCall', 'mr_strNewTree');

    // Add move subtree function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_movesubtree.AfterCall', 'mr_strMoveSubtree');

    // Add new category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_newcat.AfterCall', 'mr_strNewCategory');

    // Add rename category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_renamecat.AfterCall', 'mr_strRenameCategory');

    // Add move up category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_moveupcat.AfterCall', 'mr_strMoveUpCategory');

    // Add move down category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_movedowncat.AfterCall', 'mr_strMovedownCategory');

    // Add save article (new and existing category) function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.con_saveart.AfterCall', 'mr_conSaveArticle');

    // Add mr related function for hook "after plugins loaded" to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Frontend.AfterLoadPlugins', 'mr_runFrontendController');

    ////////////////////////////////////////////////////////////////////////////////////////////////


    // Add move article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conMoveArticles_Loop', 'mr_conMoveArticles');

    // Add duplicate article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conCopyArtLang_AfterInsert', 'mr_conCopyArtLang');

    // Add sync article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conSyncArticle_AfterInsert', 'mr_conSyncArticle');

    // Add url rewriting function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Frontend.CreateURL', 'mr_build_new_url');

    if ($aMrCfg['rewrite_urls_at_congeneratecode'] == 1) {

        // Add url rewriting at code generation to Contenido Extension Chainer
        $_cecRegistry->addChainFunction('Contenido.Content.conGenerateCode', 'mr_build_generated_code');
#        $GLOBALS['mpDebug']->addDebug('conGenerateCode()', 'added chain to', __FILE__);

    } elseif ($aMrCfg['rewrite_urls_at_front_content_output'] == 1) {

        // Add url rewriting at html output to Contenido Extension Chainer
        $_cecRegistry->addChainFunction('Contenido.Frontend.HTMLCodeOutput', 'mr_build_generated_code');
#           $GLOBALS['mpDebug']->addDebug('front_content.php', 'added chain to output at', __FILE__);

    } else {

        // Fallback solution: Add url rewriting at code generation to Contenido Extension Chainer
        $_cecRegistry->addChainFunction('Contenido.Content.conGenerateCode', 'mr_build_generated_code');
#           $GLOBALS['mpDebug']->addDebug('conGenerateCode()', 'added fallback chain to', __FILE__);

    }

}

unset($clientId, $options);
