<?php

/**
 * Mod Rewrite front_content.php controller. Does some preprocessing jobs, tries
 * to set following variables, depending on mod rewrite configuration and if
 * request part exists:
 * - $client
 * - $changeclient
 * - $lang
 * - $changelang
 * - $idart
 * - $idcat
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */


if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


$GLOBALS['mpDebug']->addDebug($cfg['mod_rewrite'], 'front_content_controller.php mod rewrite config');


global $client, $changeclient, $cfgClient, $lang, $changelang, $idart, $idcat, $path;

$aMrCfg = ModRewrite::getConfig();

// create an mod rewrite controller instance and execute processing
$oMRController = new ModRewriteController($_SERVER['REQUEST_URI']);
$oMRController->execute();

if ($oMRController->errorOccured()) {

    // an error occured (idcat and or idart couldn't catched by controller)
    // try to redirect to errorpage if desired
    if ($aMrCfg['redirect_invalid_article_to_errorsite'] == 1 && (int) $client > 0 && (int) $lang > 0) {
        global $errsite_idcat, $errsite_idart;

        if ($cfgClient['set'] != 'set')	{
            rereadClients();
        }

        // errorpage
        $errurl  = "front_content.php?client=$client&idcat=".$errsite_idcat[$client]."&idart=".$errsite_idart[$client]."&lang=$lang&error=1";
        $errsite = 'Location: ' . CEC_Hook::execute('Contenido.Frontend.CreateURL', $errurl);
        mr_header($errsite);
        exit();
    }


} else {

    // set some global variables

    if ($oMRController->getClient()) {
        $client = $oMRController->getClient();
    }

    if ($oMRController->getChangeClient()) {
        $changeclient = $oMRController->getChangeClient();
    }

    if ($oMRController->getLang()) {
        $lang = $oMRController->getLang();
    }

    if ($oMRController->getChangeLang()) {
        $changelang = $oMRController->getChangeLang();
    }

    if ($oMRController->getIdArt()) {
        $idart = $oMRController->getIdArt();
    }

    if ($oMRController->getIdCat()) {
        $idcat = $oMRController->getIdCat();
    }

    if ($oMRController->getPath()) {
        $path = $oMRController->getPath();
    }

}

// some debugs
$GLOBALS['mpDebug']->addDebug($mr_preprocessedPageError, 'mr $mr_preprocessedPageError', __FILE__);
$GLOBALS['mpDebug']->addDebug($idart, 'mr $idart', __FILE__);
$GLOBALS['mpDebug']->addDebug($idcat, 'mr $idcat', __FILE__);
$GLOBALS['mpDebug']->addDebug($lang, 'mr $lang', __FILE__);
$GLOBALS['mpDebug']->addDebug($client, 'mr $client', __FILE__);



##########################################################################################

/**
 * Replaces calling of header method for redirects in front_content.php,
 * used during development.
 *
 * @param  $header  Header value for redirect
 */
function mr_header($header) {
    header($header);return;

    $header = str_replace('Location: ', '', $header);
    echo '<html>
        <head></head>
        <body>
        <p><a href="'.$header.'">'.$header.'</a></p>';
    mr_debugOutput();
    echo '</body></html>';
    exit();
}

/**
 * Debug output only during development
 */
function mr_debugOutput($print=true) {
    if (is_array($GLOBALS['DB_Contenido_QueryCache']) && count($GLOBALS['DB_Contenido_QueryCache']) > 0) {
        $GLOBALS['mpDebug']->addDebug($GLOBALS['DB_Contenido_QueryCache'], 'sql statements');
    }

    if ($print == true) {
        echo $GLOBALS['mpDebug']->getResults();
    } else {
        return $GLOBALS['mpDebug']->getResults(false);
    }
}
