<?php
/**
 * Defines the 'modrewrite' related functions
 *
 * TODO: Rename functions/variables use coding guidelines
 *
 * @author      Stefan Seifarth / stese
 * @copyright    www.polycoder.de
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  ModRewrite
 */

/******************************************
 * File      :   functions.mod_rewrite.php
 * Project   :   Contenido
 * Descr     :   Defines the 'modrewrite' related
 *               functions
 *
 * Author    :   Stefan Seifarth
 * Created   :   04.12.2004
 * Modified  :   18.12.2005
 *
 *  www.polycoder.de
 ******************************************/


if(!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


cInclude('classes', 'contenido/class.articlelanguage.php');


/**
 * Get language id from language name thanks to Nicolas Dickinson for multi
 * Client/Language BugFix
 *
 * @param  string   Language name
 * @return integer  Language id
 */
function mr_get_language_id($str_languagename="", $int_client=1) {
    global $cfg;

    $int_lang_id = false;

    $sql = "SELECT l.idlang
           FROM " . $cfg["tab"]["lang"] . " as l
           LEFT JOIN " . $cfg["tab"]["clients_lang"] . " AS cl ON l.idlang = cl.idlang
           WHERE
              cl.idclient = '". (int)$int_client . "' AND
              l.name = '" . urldecode($str_languagename) . "'";
    if ($aData = mr_query_n_next_record($sql)) {
        $int_lang_id = $aData['idlang'];
    }

    return $int_lang_id;
}


/**
 * Get client id from client name
 *
 * @param   string   Client name
 * @return  integer  Client id
 */
function mr_get_client_id($str_clientname="") {
    global $cfg;

    $int_client_id = false;

    $sql = "SELECT idclient
            FROM " . $cfg["tab"]["clients"] . "
            WHERE name = '" . urldecode($str_clientname) . "'";
    if ($aData = mr_query_n_next_record($sql)) {
        $int_client_id = $aData['idclient'];
    }

    return $int_client_id;
}


/**
 * Get article id by article websafe name
 *
 * @param   string   Websafe name
 * @param   integer  Category id
 * @return  integer  Recent article id
 */
function mr_get_idart($str_artname="", $int_id=0, $int_lang_id=0) {
    global $cfg;
    static $db;

    $int_idart = false;

    if (!isset($db)) {
        $db = new DB_Contenido;
    }

    $str_where = "";
    // only article name were given
    if ($int_id == 0) {
        // get all basic category ids with parentid=0
        $sql = "SELECT idcat FROM " . $cfg["tab"]["cat"] . " WHERE parentid = '0'";
        $db->query ( $sql );

        $arr_idcats = array();
        $arr_where = array();

        while ($db->next_record()) {
            $arr_idcats[] = "idcat = '" . $db->f("idcat") . "'";
        }
        $str_where = " AND ( " . join(" OR ", $arr_idcats) . ")";
    } else {
        $str_where = " AND ca.idcat = '$int_id'";
    }

    $sql = "SELECT al.idart
            FROM " . $cfg["tab"]["art_lang"] . " al
            LEFT JOIN " . $cfg["tab"]["cat_art"] . " ca
    ON al.idart = ca.idart
    WHERE al.urlname = '$str_artname'" . $str_where;

    if ($aData = mr_query_n_next_record($sql)) {
        $int_idart = $aData['idart'];
    }

    return $int_idart;
}


////////////////////////////////////////////////////////////////////////////////////////////////////


/**
 * Processes mod_rewrite related job for created new tree.
 *
 * Will be called by chain 'Contenido.Action.str_newtree.AfterCall'.
 *
 * @param   array  $data  Assoziative array with some values
 * @return  array  Passed parameter
 */
function mr_strNewTree(array $data){
    $GLOBALS['mpDebug']->log($data, 'mr_strNewTree $data');

    if ((int) $data['newcategoryid'] > 0) {
        $mrCatAlias = (trim($data['categoryalias']) !== '') ? trim($data['categoryalias']) : trim($data['categoryname']);
        // set new urlname - because original set urlname isn''t validated for double entries in same parent category
        ModRewrite::set_category($mrCatAlias, $data['newcategoryid'], $GLOBALDS['lang']);
    }

    return $data;
}


/**
 * Processes mod_rewrite related job for created new category.
 *
 * Will be called by chain 'Contenido.Action.str_newcat.AfterCall'.
 *
 * @param   array  $data  Assoziative array with some values
 * @return  array  Passed parameter
 */
function mr_strNewCategory(array $data){
    $GLOBALS['mpDebug']->log($data, 'mr_strNewCategory $data');

    if ((int) $data['newcategoryid'] > 0) {
        $mrCatAlias = (trim($data['categoryalias']) !== '') ? trim($data['categoryalias']) : trim($data['categoryname']);
        // set new urlname - because original set urlname isn''t validated for double entries in same parent category
        ModRewrite::set_category($mrCatAlias, $data['newcategoryid'], $GLOBALS['lang']);
    }

    return $data;
}


/**
 * Processes mod_rewrite related job for renamed category.
 *
 * Will be called by chain 'Contenido.Action.str_renamecat.AfterCall'.
 *
 * @param   array  $data  Assoziative array with some values
 * @return  array  Passed parameter
 */
function mr_strRenameCategory(array $data){
    $GLOBALS['mpDebug']->log($data, 'mr_strRenameCategory $data');

    $mrCatAlias = (trim($data['categoryalias']) !== '') ? trim($data['categoryalias']) : trim($data['categoryname']);
    if ($mrCatAlias != '') {
        // set new urlname - because original set urlname isn''t validated for double entries in same parent category
        ModRewrite::set_category($mrCatAlias, $data['idcat'], $data['lang']);
    }

    return $data;
}


/**
 * Processes mod_rewrite related job after moving a category up.
 *
 * Will be called by chain 'Contenido.Action.str_moveupcat.AfterCall'.
 *
 * TODO: do we really need processing of the category? no mr relevant data
 * changes while moving
 *
 * @param   int  $idcat  Category id
 * @return  int  Category id
 */
function mr_strMoveUpCategory($idcat) {
    $GLOBALS['mpDebug']->log($idcat, 'mr_strMoveUpCategory $idcat');

    // category check
    $cat = new cApiCategory((int) $idcat);
    if (!$cat->get('preid')) {
        return;
    }

    // get all cat languages
    $aIdLang = ModRewrite::get_cat_languages($idcat);

    // update ...
    foreach ($aIdLang as $iIdLang) {
        // get urlname
        $str_catname = ModRewrite::get_catname($idcat, $iIdLang);
        // set new urlname - because original set urlname isn't validated for double entries in same parent category
        ModRewrite::set_category($str_catname, $idcat, $iIdLang);
    }

    return $idcat;
}


/**
 * Processes mod_rewrite related job after moving a category down.
 *
 * Will be called by chain 'Contenido.Action.str_movedowncat.AfterCall'.
 *
 * TODO: do we really need processing of the category? no mr relevant data
 * changes while moving
 *
 * @param   int  $idcat  Id of category beeing moved down
 * @return  int  Category id
 */
function mr_strMovedownCategory($idcat) {
    $GLOBALS['mpDebug']->log($idcat, 'mr_strMovedownCategory $idcat');

    // category check
    $cat = new cApiCategory((int) $idcat);
    if (!$cat->get('id')) {
        return;
    }

    // get all cat languages
    $aIdLang = ModRewrite::get_cat_languages($idcat);
    // update ...
    foreach ($aIdLang as $iIdLang) {
        // get urlname
        $sCatname = ModRewrite::get_catname($idcat, $iIdLang);
        // set new urlname - because original set urlname isn't validated for double entries in same parent category
        ModRewrite::set_category($sCatname, $idcat, $iIdLang);
    }

    return $idcat;
}


/**
 * Processes mod_rewrite related job after moving a category subtree.
 *
 * Will be called by chain 'Contenido.Action.str_movesubtree.AfterCall'.
 *
 * @param   int  $idcat  Id of category where the subtree has beeen moved
 * @return  int  Category id
 */
function mr_strMoveSubtree($idcat) {
    $GLOBALS['mpDebug']->log($idcat, 'mr_strMovedownCategory $idcat');

    // category check
    $cat = new cApiCategory($idcat);
    if (!$cat->get('id')) {
        return;
    }

    // get all cat languages
    $aIdLang = ModRewrite::get_cat_languages($idcat);
    // update ...
    foreach ($aIdLang as $iIdLang) {
        // get urlname
        $sCatname = ModRewrite::get_catname($idcat, $iIdLang);
        // set new urlname - because original set urlname isn't validated for double entries in same parent category
        ModRewrite::set_category($sCatname, $idcat, $iIdLang);
    }

    return $idcat;
}


/**
 * Processes mod_rewrite related job for saved articles (new or modified article).
 *
 * Will be called by chain 'Contenido.Action.con_saveart.AfterCall'.
 *
 * @param   array  $data  Assoziative array with some article properties
 * @return  array  Passed parameter
 */
function mr_conSaveArticle(array $data) {
    $GLOBALS['mpDebug']->log($data, 'mr_conSaveArticle $data');

//    if (!isset($title) || ((int) $idart == 0)) {
    if ((int) $data['idart'] == 0) {
        return $data;
    }

    if (strlen(trim($data['urlname'])) == 0) {
        $data['urlname'] = $data['title'];
    }

    if (1 == $GLOBALS['tmp_firstedit'])	{
        // new article
        $aLanguages = getLanguagesByClient($GLOBALS['client']);

        foreach ($aLanguages as $iLang) {
            ModRewrite::set_article($data['urlname'], $data['idart'], $iLang, $data['idcat']);
        }

    } else {
        // modified article
        $aArticle = ModRewrite::get_id_from_idartlang($data['idartlang']);

        if (isset($aArticle['idart']) && isset($aArticle['idlang'])) {
            ModRewrite::set_article($data['urlname'], $aArticle['idart'], $aArticle['idlang'], $data['idcat']);
        }

    }

    return $data;

}


////////////////////////////////////////////////////////////////////////////////////////////////////
##%%## is to remove
/**
 * Processes mod_rewrite related job after editing a article first time.
 * Will be called as a action code from table con_actions.
 * @param  int  $newIdart  Id of new edited article
 */
/*
function mr_conEditFirstTime($newIdart) {
    global $client, $title, $urlname, $idcat;
    if (strlen(trim($urlname)) == 0) {
        $urlname = $title;
    }
    $a_languages = getLanguagesByClient($client);
    foreach ($a_languages as $tmp_lang) {
        ModRewrite::set_article($urlname, $newIdart, $tmp_lang, $idcat);
    }
}
*/
/**
 * Processes mod_rewrite related job after editing a article.
 * Will be called as a action code from table con_actions.
 * @param  int  $idart  Id of edited article
 */
/*
function mr_conEditArt($idart){
    global $urlname, $title, $idartlang, $idcat;
    if (strlen(trim($urlname)) == 0) {
        $urlname = $title;
    }
    $arr_art = ModRewrite::get_id_from_idartlang($idartlang);
    if (isset($arr_art['idart']) && isset($arr_art['idlang'])) {
        ModRewrite::set_article($urlname, $arr_art['idart'], $arr_art['idlang'], $idcat);
    }
}
*/
////////////////////////////////////////////////////////////////////////////////////////////////////


/**
 * Processes mod_rewrite related job for articles beeing moved.
 *
 * Will be called by chain 'Contenido.Article.conMoveArticles_Loop'.
 *
 * @param   array  $data  Assoziative array with record entries
 * @return  array  Loop through of arguments
 */
function mr_conMoveArticles($data){
    $GLOBALS['mpDebug']->log($data, 'mr_conMoveArticles $data');

    // too defensive but secure way
    if (!is_array($data)) {
        return $data;
    } elseif (!isset($data['idartlang'])) {
        return $data;
    } elseif (!isset($data['idart'])) {
        return $data;
    }

    $arr_art = ModRewrite::get_artids($data['idartlang']);
    if (count($arr_art) == 2) {
        ModRewrite::set_article($arr_art["urlname"], $data['idart'], $arr_art["idlang"]);
    }

    return $data;
}


/**
 * Processes mod_rewrite related job for duplicated articles.
 *
 * Will be called by chain 'Contenido.Article.conCopyArtLang_AfterInsert'.
 *
 * @param   array  $data  Assoziative array with record entries
 * @return  array  Loop through of arguments
 */
function mr_conCopyArtLang($data) {
    $GLOBALS['mpDebug']->log($data, 'mr_conCopyArtLang $data');

    // too defensive but secure way
    if (!is_array($data)) {
        return $data;
    } elseif (!isset($data['title'])) {
        return $data;
    } elseif (!isset($data['idart'])) {
        return $data;
    } elseif (!isset($data['idlang'])) {
        return $data;
    }

    ModRewrite::set_article($data['title'], $data['idart'], $data['idlang']);

    return $data;
}


/**
 * Processes mod_rewrite related job for synchronized articles.
 *
 * Will be called by chain 'Contenido.Article.conSyncArticle_AfterInsert'.
 *
 * @param   array  $data  Assoziative array with record entries as follows:
 * <code>
 * array(
 *     'src_art_lang'  => Recordset (assoziative array) of source item from con_art_lang table
 *     'dest_art_lang' => Recordset (assoziative array) of inserted destination item from con_art_lang table
 * );
 * </code>
 *
 * @return  array  Loop through of argument
 */
function mr_conSyncArticle($data){
    $GLOBALS['mpDebug']->log($data, 'mr_conSyncArticle $data');

    // too defensive but secure way
    if (!is_array($data)) {
        return $data;
    } elseif (!isset($data['src_art_lang']) || !is_array($data['src_art_lang'])) {
        return $data;
    } elseif (!isset($data['dest_art_lang']) || !is_array($data['dest_art_lang'])) {
        return $data;
    } elseif (!isset($data['dest_art_lang']['idart'])) {
        return $data;
    } elseif (!isset($data['dest_art_lang']['idlang'])) {
        return $data;
    }

    if (!isset($data['src_art_lang']['urlname'])) {
        $artLang = new cApiArticleLanguage($data['src_art_lang']['idartlang']);
        $urlname = $artLang->get('urlname');
    } else {
        $urlname = $data['src_art_lang']['urlname'];
    }

    if ($urlname) {
        ModRewrite::set_article($urlname, $data['dest_art_lang']['idart'], $data['dest_art_lang']['idlang']);
    }

    return $data;
}


/**
 *
 * Works as a wrapper for creation of urls using Contenido_UrlBuilder_MR.
 *
 * Will be called by chain 'Contenido.Frontend.CreateURL'.
 *
 * @deprecated  Some parts of the function are replaced against new Url building
 *              logic.
 *
 * @param   string  $url  URL to rebuild
 * @return  string        New URL
 */
function mr_build_new_url($url) {

    static $mrUrlBuilder;
    if (!isset($mrUrlBuilder)) {
        cInclude('classes', 'UrlBuilder/Contenido_UrlBuilder_MR.class.php');
        $mrUrlBuilder = Contenido_UrlBuilder_MR::getInstance();
    }

    $mrUrlBuilder->buildUrl(array($url));
    $newUrl = $mrUrlBuilder->getUrl();

    $urlDebug['in']  = $url;
    $urlDebug['out'] = $newUrl;
    $GLOBALS['mpDebug']->addDebug($urlDebug, 'mr_build_new_url() in -> out');

    return $newUrl;

}


/**
 * Replaces existing ancors inside passed code, while rebuilding the urls.
 *
 * Will be called by chain 'Contenido.Content.conGenerateCode' or
 * 'Contenido.Frontend.HTMLCodeOutput' depening on mod_rewrite settings.
 *
 * @param   string  $code   Code to prepare
 * @return  string          New code
 */
function mr_build_generated_code($code) {
    global $cfg;

    $GLOBALS['mpDebug']->addDebug($code, 'mr_build_generated_code() in');

    // mod rewrite is activated
    if (ModRewrite::is_enabled()) {
        $sseStarttime = getmicrotime();

        // edit 060603 - anchor hack
        $code = preg_replace_callback(
            "/<a([^>]*)href\s*=\s*[\"|\'][\/]#(.?|.+?)[\"|\']([^>]*)>/i",
            create_function('$arr_matches' , 'return ModRewrite::rewrite_html_anchor($arr_matches);'),
            $code
        );

        // remove fucking tinymce single quote entities:
        $code = str_replace("&#39;", "'", $code);

        // get base uri
        $sBaseUri = $GLOBALS['cfgClient'][$GLOBALS['client']]['path']['htmlpath'];
        $sBaseUri = CEC_Hook::execute("Contenido.Frontend.BaseHrefGeneration", $sBaseUri);

        // IE hack with wrong base href interpretation
        $code = preg_replace("/([\"|\'|=])upload\/(.?|.+?)([\"|\'|>])/ie", "stripslashes('\\1{$sBaseUri}upload/\\2\\3')", $code);

        // define some preparations to replace /front_content.php & ./front_content.php
        // against front_content.php, because urls should start with front_content.php
        $aPattern = array(
            '/([\"|\'|=])\/front_content\.php(.?|.+?)([\"|\'|>])/i',
            '/([\"|\'|=])\.\/front_content\.php(.?|.+?)([\"|\'|>])/i'
        );

        $aReplace = array(
            '\1front_content.php\2\3',
            '\1front_content.php\2\3'
        );

        // perform the pre replacements
        $code = preg_replace($aPattern, $aReplace, $code);

        // ok let it beginn, start mod rewrite class
        $code = str_replace('"front_content.php"', '"' . mr_build_new_url('front_content.php') . '"', $code);
        $code = str_replace("'front_content.php'", "'" . mr_build_new_url('front_content.php') . "'", $code);
        $code = preg_replace_callback(
            "/([\"|\'|=])front_content\.php(.?|.+?)([\"|\'|>])/i",
            create_function('$arr_matches' , 'return $arr_matches[1] . mr_build_new_url("front_content.php" . $arr_matches[2]) . $arr_matches[3];'),
            $code
        );

        $sseEndtime = getmicrotime();
    } else {
        // anchor hack for non modrewrite websites
        $code = preg_replace_callback(
                    "/<a([^>]*)href\s*=\s*[\"|\'][\/]#(.?|.+?)[\"|\']([^>]*)>/i",
        create_function('$arr_matches' , 'return ModRewrite::contenido_html_anchor($arr_matches, $GLOBALS["is_XHTML"]);'),
        $code
        );
    }

    $GLOBALS['mpDebug']->addDebug($code, 'mr_build_generated_code() out');

    if ($debug = mr_debugOutput(false)) {
        $code = str_ireplace_once("</body>", $debug . "\n</body>", $code);
    }

    return $code;
    // print "\n\n<!-- modrewrite generation time: " . ($sseEndtime - $sseStarttime) . " seconds -->";
}


/**
 * Sets language of client, like done in front_content.php
 *
 * @param  int  $client  Client id
 */
function mr_set_client_language($client) {
    if ($GLOBALS['lang'] || (int) $GLOBALS['lang'] > 0) {
        // there is nothing to do
        return;
    }

    // use the first language of this client
    if (isset($GLOBALS['load_lang'])) {
        // load_client is set in frontend/config.php
        $GLOBALS['lang'] = $GLOBALS['load_lang'];
        return;
    }

    // try to get clients language from table
    $sql = "SELECT B.idlang FROM
                ".$GLOBALS['cfg']['tab']['clients_lang']." AS A,
                ".$GLOBALS['cfg']['tab']['lang']." AS B
            WHERE
                A.idclient='" . ((int) $client) . "' AND
                A.idlang=B.idlang
            LIMIT 0,1";

    if ($aData = mr_query_n_next_record($sql)) {
        $GLOBALS['lang'] = $aData['idlang'];
    }
}


/**
 * Loads Advanced Mod Rewrite configuration for passed client using serialized
 * file containing the settings.
 *
 * File is placed in /contenido/mod_rewrite/includes/and is named like
 * config.mod_rewrite_{client_id}.php.
 *
 * @param  int  $clientId  Id of client
 */
function mr_loadConfiguration($clientId) {
    global $cfg;
    static $aLoaded;

    $clientId = (int) $clientId;
    if (!isset($aLoaded)) {
        $aLoaded = array();
    } elseif (isset($aLoaded[$clientId])) {
        return;
    }

    $options['key'] = $cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/config.mod_rewrite_' . $clientId . '.php';

    $config = ConfigFactory::get('serializer', $options);
    $mrConfig = $config->get();
    if (is_array($mrConfig)) {

        // merge mod rewrite configuration with global cfg array
        $cfg = array_merge($cfg, $mrConfig);

    } else {

        // couldn't load configuration, set defaults
        include_once($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/config.mod_rewrite_default.php');

    }

    $aLoaded[$clientId] = true;

}


/**
 * Includes the frontend controller script which parses the url and extacts
 * needed data like idcat, idart, lang and client from it.
 *
 * Will be called by chain 'Contenido.Frontend.AfterLoadPlugins' at front_content.php.
 *
 * @return  bool  Just a return value
 */
function mr_runFrontendController() {
    $iStartTime = getmicrotime();

    plugin_include('mod_rewrite', 'includes/config.plugin.php');

    if (ModRewrite::is_enabled() == true) {

        plugin_include('mod_rewrite', 'includes/front_content_controller.php');

        $totalTime = sprintf('%.4f', (getmicrotime() - $iStartTime));
        $GLOBALS['mpDebug']->addDebug($totalTime, 'mr_runFrontendController() total time');

    }

    return true;
}


/**
 * Funcion to return cat id by path. Loads cached data structure about resolved
 * paths only once.
 *
 * @param   string  $path  Category path
 * @return  int  Category id
 */
function mr_prResolvePathViaURLNames($path){
    global $cfg, $client, $lang;
    static $aLoaded;

    // TODO: better handling of resolved paths. store in cat_lang.urlpathnamne

    $filename = 'cache.mod_rewrite_path_resolver' . $client . '-' .$lang . '.php';

    $aResolvedPaths = array();

    if (!isset($aLoaded)) {
        $aLoaded = array();
    } elseif (isset($aLoaded[$filename])) {
        $aResolvedPaths = $aLoaded[$filename];
    }

    // options for data serializer
    $options = array(
        'key' => $cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/' . $filename,
        'livetime' => 86400 // 1 day
    );

    if (count($aResolvedPaths) == 0) {
        // try to get resolved path array from cache file
        $config = ConfigFactory::get('serializer', $options);
        if ($arr = $config->get()) {
            $aResolvedPaths = $arr;
        }
    }

    if (count($aResolvedPaths) == 0) {
        // resolve paths by fetching them from db

        $db = new DB_Contenido();

        // Fetch all category names, build path strings
        $sql = "SELECT * FROM "
                 . $cfg["tab"]["cat_tree"] . " AS A, "
                 . $cfg["tab"]["cat"] . "      AS B, "
                 . $cfg["tab"]["cat_lang"]."   AS C
             WHERE
                 A.idcat=B.idcat AND
                 B.idcat=C.idcat AND
                 C.idlang='" . Contenido_Security::toInteger($lang) . "' AND
                 C.visible=1 AND
                 B.idclient='" . Contenido_Security::toInteger($client) . "'
             ORDER BY A.idtree";
        $db->query($sql);

        $catsAsHTML = ModRewrite::validate_setting_categories_as_html();
        $ext = '';

        if (isset($cfg['mod_rewrite']['file_extension'])) {
            $ext = $cfg['mod_rewrite']['file_extension'];
        }

        while ($db->next_record()) {
/*
            $catPath = mr_build_new_url('front_content.php?idcat=' . $db->f('idcat'));

            if ($catsAsHTML) {
                if ($ext !== '' && substr($catPath, (0-strlen($ext))) == $ext) {
                    $catPath = substr($catPath, 0, (0-strlen($ext)));
                }
                // split categoryPath by seaparator between category and article
                list($catPart, $artPart) = explode($cfg['mod_rewrite']['article_seperator'], $catPath);
                $catPath = $catPart;
                $catPath = str_replace($cfg['mod_rewrite']['category_seperator'], '/', $catPath);
                if (substr($path, -1) == '/' && substr($catPath, -1) !== '/') {
                    $catPath .= '/';
                    $catPath = str_replace('//', '/', $catPath);
                }
            }
*/

            $catPath = $db->f('idcat');
            prCreateURLNameLocationString($db->f('idcat'), '/', $catPath, false, '', 0, 0, true, true);

            // store path
            $aResolvedPaths[$db->f('idcat')] = array(
                'path'  => $catPath,
                'name'  => $db->f('name'),
                'level' => $db->f('level')
            );
        }

        // store resolved path data into cache file
        $config = ConfigFactory::get('serializer', $options);
        if (!$config->set($aResolvedPaths)) {
            cWarning(__FILE__, __LINE__, 'Couldn\'t store resolved path data into cache file!');
        }

        $aLoaded[$filename] = $aResolvedPaths;

    }

    // compare strings using the similar_text algorithm
    $percent = 0;
    foreach ($aResolvedPaths as $id => $item) {
        similar_text($path, $item['path'], $percent);
        $results[$id] = $percent;
    }

    arsort($results, SORT_NUMERIC);
    reset($results);

    $GLOBALS['mpDebug']->addDebug($path, 'functions.pathresolver.php mr_prResolvePathViaURLNames $path');
    $GLOBALS['mpDebug']->addDebug($aResolvedPaths, 'functions.pathresolver.php mr_prResolvePathViaURLNames $aResolvedPaths');
    $GLOBALS['mpDebug']->addDebug($results, 'functions.pathresolver.php mr_prResolvePathViaURLNames $results');
    return ModRewrite::get_id_from_pathresolver_result($results);

}


function mr_resetPathResolverCache() {
    global $cfg, $client, $lang;

    $filename = 'cache.mod_rewrite_path_resolver' . $client . '-' .$lang . '.php';

    // options for data serializer
    $options = array(
        'key' => $cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mod_rewrite/includes/' . $filename,
        'livetime' => 86400 // 1 day
    );

    // try to remove cache file
    $config = ConfigFactory::get('serializer', $options);
    $config->remove();

}


################################################################################
### Some helper fuctions, which are not plugin specific


/**
 * Database query helper. Used to execute a select statement and to return the
 * result of first recordset.
 *
 * Minimizes following code:
 * <code>
 * // default way
 * $db  = new DB_Contenido();
 * $sql = "SELECT * FROM foo WHERE bar='foobar'";
 * $db->query($sql);
 * $db->next_record();
 * $data = $db->Record;
 *
 * // new way
 * $sql  = "SELECT * FROM foo WHERE bar='foobar'";
 * $data = mr_query_n_next_record($sql);
 * </code>
 *
 * @param   string  $query  Query to execute
 * @return  mixed   Assoziative array including recordset or null
 */
function mr_query_n_next_record($query){
    static $db;
    if (!isset($db)) {
        $db = new DB_Contenido();
    }
    $db->query($query);
    return ($db->next_record()) ? $db->Record : null;
}


/**
 * Returns value of an array key (assoziative or indexed).
 *
 * Shortcut function for some ways to access to arrays:
 * <code>
 * // old way
 * if (is_array($foo) && isset($foo['bar']) && $foo['bar'] == 'yieeha') {
 *     // do something
 * }
 *
 * // new, more readable way:
 * if (mr_arrayValue($foo, 'bar') == 'yieeha') {
 *     // do something
 * }
 *
 * // old way
 * if (is_array($foo) && isset($foo['bar'])) {
 *     $jep = $foo['bar'];
 * } else {
 *     $jep = 'yummy';
 * }
 *
 * // new way
 * $jep = mr_arrayValue($foo, 'bar', 'yummy');
 * </code>
 *
 * @param   array  $array    The array
 * @param   mixed  $key      Position of an indexed array or key of an assoziative array
 * @param   mixed  $default  Default value to return
 * @return  mixed  Either the found value or the default value
 */
function mr_arrayValue($array, $key, $default=null) {
    if (!is_array($array)) {
        return $default;
    } elseif (!isset($array[$key])) {
        return $default;
    } else {
        return $array[$key];
    }
}


/**
 * Request cleanup function. Request data is allways tainted and must be filtered.
 * Pass the array to cleanup using several options.
 * Emulates array_walk_recursive().
 *
 * @param   mixed  $data     Data to cleanup
 * @param   array  $options  Default options array, provides only 'filter' key with several
 *                           filter functions which are to execute as follows:
 * <code>
 * $options['filter'] = array('trim', 'myFilterFunc');
 * </code>
 *                           If no filter functions are set, 'trim', 'strip_tags' and 'stripslashes'
 *                           will be used by default.
 *                           A userdefined function must accept the value as a parameter and must return
 *                           the filtered parameter, e. g.
 * <code>
 * function myFilter($data) {
 *    // do what you want with the data, e. g. cleanup of xss content
 *    return $data;
 * }
 * </code>
 *
 * @return  mixed  Cleaned data
 */
function mr_requestCleanup(&$data, $options=null) {
    if (!mr_arrayValue($options, 'filter')) {
        $options['filter'] = array('trim', 'strip_tags', 'stripslashes');
    }

    if (is_array($data)) {
        foreach ($data as $p => $v) {
            $data[$p] = mr_requestCleanup($v, $options);
        }
    } else {
        foreach ($options['filter'] as $filter) {
            if ($filter == 'trim') {
                $data = trim($data);
            } elseif ($filter == 'strip_tags') {
                $data = strip_tags($data);
            } elseif ($filter == 'stripslashes') {
                $data = stripslashes($data);
            } elseif (function_exists($filer)) {
                $data = call_user_func($filter, $data);
            }
        }
    }
    return $data;
}
